<?php

namespace B2buy\Tutorial\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\Tutorial\Http\Requests\TutorialtagRequest;
use B2buy\Tutorial\Interfaces\TutorialtagRepositoryInterface;
use B2buy\Tutorial\Models\Tutorialtag;

/**
 * Resource controller class for tutorialtag.
 */
class TutorialtagResourceController extends BaseController
{

    /**
     * Initialize tutorialtag resource controller.
     *
     * @param type TutorialtagRepositoryInterface $tutorialtag
     *
     * @return null
     */
    public function __construct(TutorialtagRepositoryInterface $tutorialtag)
    {
        parent::__construct();
        $this->repository = $tutorialtag;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Tutorial\Repositories\Criteria\TutorialtagResourceCriteria::class);
    }

    /**
     * Display a list of tutorialtag.
     *
     * @return Response
     */
    public function index(TutorialtagRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\Tutorial\Repositories\Presenter\TutorialtagPresenter::class)
                ->$function();
        }

        $tutorialtags = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('tutorial::tutorialtag.names'))
            ->view('tutorial::tutorialtag.index', true)
            ->data(compact('tutorialtags', 'view'))
            ->output();
    }

    /**
     * Display tutorialtag.
     *
     * @param Request $request
     * @param Model   $tutorialtag
     *
     * @return Response
     */
    public function show(TutorialtagRequest $request, Tutorialtag $tutorialtag)
    {

        if ($tutorialtag->exists) {
            $view = 'tutorial::tutorialtag.show';
        } else {
            $view = 'tutorial::tutorialtag.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('tutorial::tutorialtag.name'))
            ->data(compact('tutorialtag'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new tutorialtag.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TutorialtagRequest $request)
    {

        $tutorialtag = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('tutorial::tutorialtag.name')) 
            ->view('tutorial::tutorialtag.create', true) 
            ->data(compact('tutorialtag'))
            ->output();
    }

    /**
     * Create new tutorialtag.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TutorialtagRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $tutorialtag                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('tutorial::tutorialtag.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('tutorial/tutorialtag/' . $tutorialtag->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/tutorial/tutorialtag'))
                ->redirect();
        }

    }

    /**
     * Show tutorialtag for editing.
     *
     * @param Request $request
     * @param Model   $tutorialtag
     *
     * @return Response
     */
    public function edit(TutorialtagRequest $request, Tutorialtag $tutorialtag)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('tutorial::tutorialtag.name'))
            ->view('tutorial::tutorialtag.edit', true)
            ->data(compact('tutorialtag'))
            ->output();
    }

    /**
     * Update the tutorialtag.
     *
     * @param Request $request
     * @param Model   $tutorialtag
     *
     * @return Response
     */
    public function update(TutorialtagRequest $request, Tutorialtag $tutorialtag)
    {
        try {
            $attributes = $request->all();

            $tutorialtag->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('tutorial::tutorialtag.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('tutorial/tutorialtag/' . $tutorialtag->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tutorial/tutorialtag/' . $tutorialtag->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the tutorialtag.
     *
     * @param Model   $tutorialtag
     *
     * @return Response
     */
    public function destroy(TutorialtagRequest $request, Tutorialtag $tutorialtag)
    {
        try {

            $tutorialtag->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('tutorial::tutorialtag.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('tutorial/tutorialtag/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tutorial/tutorialtag/' . $tutorialtag->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple tutorialtag.
     *
     * @param Model   $tutorialtag
     *
     * @return Response
     */
    public function delete(TutorialtagRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('tutorial::tutorialtag.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('tutorial/tutorialtag'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/tutorial/tutorialtag'))
                ->redirect();
        }

    }

    /**
     * Restore deleted tutorialtags.
     *
     * @param Model   $tutorialtag
     *
     * @return Response
     */
    public function restore(TutorialtagRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('tutorial::tutorialtag.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/tutorial/tutorialtag'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/tutorial/tutorialtag/'))
                ->redirect();
        }

    }

}
